#! /usr/bin/perl
#
# inn2diablo	Reads INNs newsfeeds and hosts.nntp file, and generates
#		dnewsfeeds, diablo.hosts and dnntpspool.ctl files.
#
# Version:	@(#)inn2diablo  1.10  06-Aug-1997  miquels@cistron.nl
#

$line = '';

if ($#ARGV != 1) {
	print STDERR "Usage: inn2diablo <newsfeeds> <hosts.nntp>\n";
	exit(1);
}

#
#	First read the newsfeeds file.
#
unless (open(FD, $ARGV[0])) {
	die "inn2diablo: $ARGV[0]: $!\n";
}
#
#	And write the dnewsfeeds and dnntpspool.ctl files.
#
unless (open(FD2, ">dnewsfeeds.new")) {
	die "inn2diablo: dnewsfeeds.new: $!\n";
}

unless (open(FD3, ">dnntpspool.ctl.new")) {
	die "inn2diablo: dnntpspool.ctl.new: $!\n";
}

while(<FD>) {
	#
	#	Skip comments.
	#
	if (/(^#|^$)/) {
		print FD2;
		next;
	}

	#
	#	Put whole entry on one line before we split the fields.
	#
	chop;
	s/^\s+//;
	if (/\\$/) {
		$cont = 1;
		s/\\$//;
	} else {
		$cont = 0;
	}
	$line = $line . $_;

	if ($cont == 0) {
		#
		#	We have the entire line, split it up.
		#
		## print "$line\n";
		($name, $group, $flags, $parm) = split(/:/, $line);
		@names = split(/[,\/]/, $name);
		$group =~ s/\/.*//;
		@groups = split(/,/, $group);

		#
		#	Try to create a label.
		#
		@parts = split(/\./, $names[0]);
		if (@parts > 1) {
			$label = $parts[$#parts - 1];
			if ($label =~ m/^(co|com|org|edu|ac|gov|net)$/) {
				$label = $parts[$#parts - 2];
			}
		} else {
			$label = $names[0];
		}
		#
		#	For label 'ME', create a 'GLOBAL' entry with
		#	just aliases if needed.
		#
		if ($label eq 'ME' && $#names >= 0) {
			print FD2 "label GLOBAL\n";
			foreach $i (@names) {
				$labels{$i} = $label;
				print FD2 "\talias $i\n";
			}
			print FD2 "end\n\n";
		}

		#
		#	If the label is 'ME', create a groupdef.
		#
		$cmd = 'label';
		if ($label eq 'ME') {
			$cmd = 'groupdef';
			$label = 'COMMON';
			$ME_seen = 1;
			unless (grep(/(^\*$|!\*$)/, @groups)) {
				unshift(@groups, '*');
			}
			@totgroups = @prefix = @groups;
		} else {
			@totgroups = (@groups, @prefix);
		}
		print FD2 "$cmd $label\n";

		#
		#	Only add aliases if this is a label.
		#
		if ($cmd eq 'label') {
			$labels{$parm} = $label if ($parm ne '');
			foreach $i (@names) {
				$labels{$i} = $label;
				print FD2 "\talias $i\n";
			}
			$xlabel{$names[0]} = $label;
		}

		#
		#	Print initial 'addgroup *' or 'groupref COMMON'
		#
		if (!grep(/(^\*$|!\*$)/, @totgroups) && $label eq 'COMMON') {
			print FD2 "\taddgroup *\n";
		} else {
			print FD2 "\tgroupref COMMON\n"
				if ($ME_seen && $label ne 'COMMON' &&
				$groups[0] ne '*' && $groups[0] ne '!*');
		}
		#
		#	Add groups one by one
		#
		foreach $i (@groups) {
			if ($i =~ m/^!/) {
				$i =~ s/^.//;
				print FD2 "\tdelgroup $i\n";
			} elsif ($i =~ m/^\@/) {
				$i =~ s/^.//;
				print FD2 "\tdelgroupany $i\n";
			} else {
				print FD2 "\taddgroup $i\n";
			}
		}
		print FD2 "end\n";
		$line = '';

		#
		#	Maybe add a line to dnntpspool.ctl
		#
		if ($names[0] =~ m/\./) {
			print FD3 tabs($label, 3);
			print FD3 tabs($names[0], 5);
			print FD3 "50\n";
		}
	}
}
close FD;
close FD2;
close FD3;

#
#	Read hosts.nntp and create diablo.hosts
#
unless (open(FD, $ARGV[1])) {
	die "inn2diablo: $ARGV[1]: $!\n";
}
unless (open(FD2, ">diablo.hosts.new")) {
	die "inn2diablo: diablo.hosts.new: $!\n";
}

#
#	Write a diablo.hosts file.
#
while(<FD>) {
	if (/(^#|^$)/) {
		print FD2;
		next;
	}

	chop;
	s/:.*//;
	$label = $xlabel{$_};
	if ($label eq '') {
		$label = $labels{$_};
	}
	print FD2;
	print FD2 "\t$label" if ($label ne '');
	print FD2 "\n";
}
close FD;
close FD2;

0;

#
#	Neat outlining of tabs for dspoolout.ctl
#
sub tabs {
	my ($word, $nt) = @_;
	my ($i, $len, $ret);

	$ret= $word;
	$len = length($word);
	for ($i = (8 * $nt) - $len; $i > 8; $i -= 8) {
		$ret .= "\t";
	}
	$ret;
}

